-------- static policy = true
CREATE OR REPLACE FUNCTION user_only (
  p_schema  IN  VARCHAR2 DEFAULT NULL,
  p_object  IN  VARCHAR2 DEFAULT NULL)
  RETURN VARCHAR2
AS
BEGIN
  -- stall for 2 seconds
  DBMS_LOCK.sleep (2);
  RETURN 'username = sys_context(''userenv'',''session_user'')';
END;
/

conn scott/tiger
set timing on
SELECT username, salary FROM people
 WHERE deptno = 20;

-- 9i feature of static policy
conn sec_mgr/oracle10g
BEGIN
  -- Remove current policy
  DBMS_RLS.drop_policy
                 (object_schema    => 'SCOTT',
                  object_name      => 'PEOPLE',
                  policy_name      => 'people_sel_sal');
  -- Add policy again but now with Caching
  DBMS_RLS.add_policy
      (object_schema            => 'SCOTT',
       object_name              => 'PEOPLE',
       policy_name              => 'people_sel_sal',
       function_schema          => 'SEC_MGR',
       policy_function          => 'user_only',
       statement_types          => 'SELECT',
       sec_relevant_cols        => 'SALARY',
       static_policy            => true);
END;
/

conn scott/tiger
SET timing on
SELECT username, salary FROM people
 WHERE deptno = 20;
-- Reconnect. Establish a different session for same user
conn scott/tiger
set timing on
SELECT username, salary FROM people
 WHERE deptno = 20;

-- Connect as a different user
conn system/manager
SET timing on
SELECT username, salary FROM scott.people
 WHERE deptno = 20;


-------------------------- context caching --------------
CREATE OR REPLACE PACKAGE people_ctx_mgr
AS
  PROCEDURE set_deptno;
  PROCEDURE set_deptno(p_deptno in number);
  PROCEDURE clear_deptno;
END;
/

CREATE OR REPLACE PACKAGE BODY people_ctx_mgr
AS
-----------------------------------------
  PROCEDURE set_deptno
  AS
    l_deptno  NUMBER;
  BEGIN
    SELECT deptno
      INTO l_deptno
      FROM lookup_dept
     WHERE username =
             SYS_CONTEXT ('userenv',
                          'session_user');
    DBMS_SESSION.set_context
                       (namespace    => 'people_ctx',
                        ATTRIBUTE    => 'deptno',
                        VALUE        => l_deptno);
  END set_deptno;
-----------------------------------------
  PROCEDURE set_deptno(p_deptno in number)
  AS
  BEGIN
    DBMS_SESSION.set_context
                       (namespace    => 'people_ctx',
                        ATTRIBUTE    => 'deptno',
                        VALUE        => p_deptno);
  END set_deptno;
-----------------------------------------
  PROCEDURE clear_deptno
  AS
  BEGIN
    DBMS_SESSION.clear_context
                      (namespace    => 'people_ctx',
                       ATTRIBUTE    => 'deptno');
  END clear_deptno;
-----------------------------------------
END people_ctx_mgr;
/
grant execute on people_ctx_mgr to scott;

-- disable all select policies for EMP, DEPT, and PEOPLE
DECLARE
  l_str  VARCHAR2 (100);
BEGIN
  FOR rec IN (SELECT *
                FROM dba_policies
               WHERE object_owner = 'SCOTT'
                 AND object_name IN
                        ('EMP', 'DEPT', 'PEOPLE')
                 AND sel = 'YES')
  LOOP
    l_str :=
         'begin DBMS_RLS.drop_policy(''SCOTT'','''
      || rec.object_name
      || ''','''
      || rec.policy_name
      || '''); end;';
    DBMS_OUTPUT.put_line (l_str);
    EXECUTE IMMEDIATE l_str;
  END LOOP;
END;
/

-- first policy on SEC_MGR's PEOPLE table
BEGIN
  DBMS_RLS.drop_policy
               (object_schema    => 'SCOTT',
                object_name      => 'PEOPLE',
                policy_name      => 'PEOPLE_SEL_CACHE');
END;
/
BEGIN
  DBMS_RLS.drop_policy
                  (object_schema    => 'SCOTT',
                   object_name      => 'EMP',
                   policy_name      => 'EMP_SEL_CACHE');
END;
/
BEGIN
  DBMS_RLS.drop_policy
                  (object_schema    => 'SCOTT',
            object_name        => 'DEPT',
            policy_name        => 'DEPT_SEL_CACHE');
END;
/
BEGIN
  -- Add policy on PEOPLE table
  DBMS_RLS.add_policy
           (object_schema      => 'SCOTT',
            object_name        => 'PEOPLE',
            policy_name        => 'PEOPLE_SEL_CACHE',
            function_schema    => 'SEC_MGR',
            policy_function    => 'PRED_FUNCTION',
            statement_types    => 'SELECT',
            policy_type        => DBMS_RLS.shared_static);
  -- Add policy on EMP table
  DBMS_RLS.add_policy
           (object_schema      => 'SCOTT',
            object_name        => 'EMP',
            policy_name        => 'EMP_SEL_CACHE',
            function_schema    => 'SEC_MGR',
            policy_function    => 'PRED_FUNCTION',
            statement_types    => 'SELECT',
            policy_type        => DBMS_RLS.shared_static);
  -- Add policy on DEPT table
  DBMS_RLS.add_policy
           (object_schema      => 'SCOTT',
            object_name        => 'DEPT',
            policy_name        => 'DEPT_SEL_CACHE',
            function_schema    => 'SEC_MGR',
            policy_function    => 'PRED_FUNCTION',
            statement_types    => 'SELECT',
            policy_type        => DBMS_RLS.shared_static);
END;
/
/********
BEGIN
  DBMS_RLS.add_policy
           (object_schema      => 'SCOTT',
            object_name        => 'EMP_copy',
            policy_name        => 'ec_SEL_CACHE',
            function_schema    => 'SEC_MGR',
            policy_function    => 'PRED_FUNCTION',
            statement_types    => 'SELECT',
            policy_type        => DBMS_RLS.shared_static);
END;
/
********/

conn scott/tiger
SET timing on
-- this should take 4 seconds:
-- 2 sec for parse, and 2 for execution
SELECT COUNT (*) FROM emp;
-- run again to see if predicate is cached
/

-- Query a different table that uses same policy function
-- The database should use the cached predicate
SELECT COUNT (*) FROM people;

-- Query a third table that uses same policy function
-- The database should use the cached predicate
SELECT COUNT (*) FROM dept;
SELECT COUNT (*) FROM emp_copy;

CONN scott/tiger
SET timing on
SELECT ename
  FROM scott.emp;
SELECT ename
  FROM sec_mgr.people;

---------- Caution ------------
-------- static policy = true

BEGIN
  DBMS_RLS.drop_policy
                      (object_schema    => 'SCOTT',
                       object_name      => 'EMP',
                       policy_name      => 'SEL_CACHE');
END;
/

-- Wrong implementation for caching
CREATE OR REPLACE FUNCTION pred_function_9_to_5 (
  p_schema  IN  VARCHAR2 DEFAULT NULL,
  p_object  IN  VARCHAR2 DEFAULT NULL)
  RETURN VARCHAR2
AS
BEGIN
  IF TO_CHAR (SYSDATE, 'HH24') BETWEEN 9 AND 17
  THEN
    RETURN '1=1';
  ELSE
    RETURN '1=0';
  END IF;
END;
/

BEGIN
  -- Add policy on PEOPLE table
  DBMS_RLS.add_policy
             (object_schema      => 'SCOTT',
              object_name        => 'emp',
              policy_name        => 'SEL_CACHE',
              function_schema    => 'SEC_MGR',
              policy_function    => 'pred_function_9_to_5',
              statement_types    => 'SELECT',
              policy_type        => DBMS_RLS.STATIC);
END;
/
SELECT TO_CHAR (SYSDATE, 'HH24') FROM dual;
SELECT COUNT (*) FROM scott.emp;


BEGIN
  DBMS_RLS.drop_policy
                      (object_schema    => 'SCOTT',
                       object_name      => 'EMP',
                       policy_name      => 'SEL_CACHE');
END;
/

-- Correct implementation for caching.
-- Performance could be a factor as evaluation is
-- done for each row
CREATE OR REPLACE FUNCTION pred_function_9_to_5 (
  p_schema  IN  VARCHAR2 DEFAULT NULL,
  p_object  IN  VARCHAR2 DEFAULT NULL)
  RETURN VARCHAR2
AS
BEGIN
  RETURN 'to_char(sysdate,''HH24'') between 9 and 17';
END;
/

BEGIN
  -- Add policy on PEOPLE table
  DBMS_RLS.add_policy
             (object_schema      => 'SCOTT',
              object_name        => 'emp',
              policy_name        => 'SEL_CACHE',
              function_schema    => 'SEC_MGR',
              policy_function    => 'pred_function_9_to_5',
              statement_types    => 'SELECT',
              policy_type        => DBMS_RLS.STATIC);
END;
/
SELECT TO_CHAR (SYSDATE, 'HH24') FROM dual;
SELECT COUNT (*) FROM scott.emp;
