-- create table for announcments example
CONN scott/tiger
DROP TABLE announcements;
CREATE TABLE announcements
(MESSAGE     VARCHAR2(4000));


CONN lbacsys/lbacsys
REM ====================================================================
REM  Step 1: Create the Policy.
REM
REM ====================================================================
EXEC sa_sysdba.drop_policy(policy_name => 'ESBD');

BEGIN
  sa_sysdba.create_policy
                       (policy_name    => 'ESBD',
                        column_name    => 'rowlabel');
END;
/

REM ====================================================================
REM  Step 2: Delegated Administration Grants
REM
REM  Automatic role is created. Need to grant privileges
REM  to allow for administration of ESBD Policy only
REM ====================================================================
-- Privileges and authorization to administer the ESBD policy
GRANT esbd_dba TO sec_mgr;

------------ Least Privileges for OLS Admin ----------
-- Privileges to create components that make up valid labels
GRANT EXECUTE ON sa_components  TO sec_mgr;
-- Privileges to create the valid labels
GRANT EXECUTE ON sa_label_admin TO sec_mgr;
-- Privileges to assign authorization labels to users
GRANT EXECUTE ON sa_user_admin  TO sec_mgr;
-- Privileges to convert a character string to 
-- its numeric label representation
GRANT EXECUTE ON char_to_label  TO sec_mgr;

-- Create a new OLS Policy, but don't assign privileges
BEGIN
  sa_sysdba.create_policy
                (policy_name    => 'Different_Policy');
END;
/
CONN sec_mgr/oracle10g
/***
  Try to administer the new policy.
  This will fail because the user does not have role authorization.
**/
BEGIN
  sa_components.create_level
               (policy_name    => 'Different_Policy',
                long_name      => 'foo',
                short_name     => 'bar',
                level_num      => 9);
END;
/

conn lbacsys/lbacsys


CONN sec_mgr/oracle10g
REM ====================================================================
REM Step 2: Define the Components of the Labels.
REM
REM Define the levels, compartments, and groups which form the components
REM of the 'ESBD' policy's labels.
REM

REM ====================================================================
REM  Creating Levels, Compartments and Groups for 'ESBD' Policy
REM ====================================================================

BEGIN
  -- Create the highest level for the company executives
  sa_components.create_level
                 (policy_name    => 'ESBD',
                  long_name      => 'Executive Staff',
                  short_name     => 'EXEC',
                  level_num      => 9000);
  -- Create the manager level
  sa_components.create_level
                          (policy_name    => 'ESBD',
                           long_name      => 'Manager',
                           short_name     => 'MGR',
                           level_num      => 8000);
  -- Create the employee level
  sa_components.create_level
                         (policy_name    => 'ESBD',
                          long_name      => 'Employee',
                          short_name     => 'EMP',
                          level_num      => 7000);
END;
/


-- create labels based on levels
BEGIN
  -- Create a label for the executives
  sa_label_admin.create_label
                          (policy_name    => 'ESBD',
                           label_tag      => 1,
                           label_value    => 'EXEC');
  -- Create a label for the management level
  sa_label_admin.create_label
                           (policy_name    => 'ESBD',
                            label_tag      => 2,
                            label_value    => 'MGR');
  -- Create a label for the employees
  sa_label_admin.create_label
                           (policy_name    => 'ESBD',
                            label_tag      => 3,
                            label_value    => 'EMP');
END;
/

-- Place some data into the data table
CONN scott/tiger
INSERT INTO announcements
     VALUES ('This message is only for the Executive Staff.');
INSERT INTO announcements
     VALUES ('All Managers: employee compensation announcement...');
INSERT INTO announcements
     VALUES ('This message is to notify all employees...');
COMMIT ;


-- place label column on table
CONN sec_mgr/oracle10g

BEGIN
  sa_policy_admin.apply_table_policy
                  (policy_name      => 'ESBD',
                   schema_name      => 'SCOTT',
                   table_name       => 'ANNOUNCEMENTS',
                   table_options    => 'NO_CONTROL');
END;
/

CONN scott/tiger
-- note the table has changed
DESC announcements
GRANT SELECT, INSERT, UPDATE 
  ON announcements TO sec_mgr;

---------- set the labels
-- Set all records to lowest level
UPDATE scott.announcements
   SET ROWLABEL = char_to_label ('ESBD', 'EMP');
-- Increase level for manager's records
UPDATE scott.announcements
   SET ROWLABEL = char_to_label ('ESBD', 'MGR')
 WHERE UPPER (MESSAGE) LIKE '%MANAGE%';
-- Increase level for manager's records
UPDATE scott.announcements
   SET ROWLABEL = char_to_label ('ESBD', 'EXEC')
 WHERE UPPER (MESSAGE) LIKE '%EXECUTIVE%';
COMMIT ;


---------- setup authorizations
BEGIN
  sa_user_admin.set_user_labels
                   (policy_name       => 'ESBD',
                    user_name         => 'ALL_EMPLOYEES',
                    max_read_label    => 'EMP');
  sa_user_admin.set_user_labels
                     (policy_name       => 'ESBD',
                      user_name         => 'ALL_MANAGERS',
                      max_read_label    => 'MGR');
  sa_user_admin.set_user_labels
                        (policy_name       => 'ESBD',
                         user_name         => 'ALL_EXECS',
                         max_read_label    => 'EXEC');
END;
/


---------- enforce read_control
BEGIN
  sa_policy_admin.remove_table_policy
                   (policy_name    => 'ESBD',
                    schema_name    => 'SCOTT',
                    table_name     => 'ANNOUNCEMENTS');
  sa_policy_admin.apply_table_policy
                  (policy_name      => 'ESBD',
                   schema_name      => 'SCOTT',
                   table_name       => 'ANNOUNCEMENTS',
                   table_options    => 'READ_CONTROL');
END;
/

-- give sec_mgr privs to test levels
BEGIN
  sa_user_admin.set_user_privs
                  (policy_name    => 'ESBD',
                   user_name      => 'SEC_MGR',
                   PRIVILEGES     => 'PROFILE_ACCESS');
END;
/

-------- Test the security ---------
CONN sec_mgr/oracle10g
COL message format a63
-- Set the authorization to the employees
BEGIN
  sa_session.set_access_profile ('ESBD',
                                 'ALL_EMPLOYEES');
END;
/
SELECT MESSAGE FROM scott.announcements;
-- Set the authorization now to be the managers
BEGIN
  sa_session.set_access_profile ('ESBD',
                                 'ALL_MANAGERS');
END;
/
SELECT MESSAGE FROM scott.announcements;
-- Set the authorization to be the executive staff
BEGIN
  sa_session.set_access_profile ('ESBD',
                                 'ALL_EXECS');
END;
/
SELECT MESSAGE FROM scott.announcements;

------------ Shoe the read labels ------------
-- Set the Label to executives
BEGIN
  sa_session.set_access_profile ('ESBD',
                                 'ALL_EXECS');
END;
/
-- Verify the label
COL "Read Label" format a25
SELECT sa_session.read_label ('ESBD') "Read Label"
  FROM DUAL;

-------------- compartments -------------------

BEGIN
  sa_components.create_compartment
                   (policy_name    => 'ESBD',
                    long_name      => 'Product Sales',
                    short_name     => 'SALES',
                    comp_num       => 1000);
  sa_components.create_compartment
              (policy_name    => 'ESBD',
               long_name      => 'Product Development',
               short_name     => 'DEV',
               comp_num       => 100);
  sa_components.create_compartment
                 (policy_name    => 'ESBD',
                  long_name      => 'Internal Support',
                  short_name     => 'IS',
                  comp_num       => 10);
END;
/

-- create compartment based labels

BEGIN
  sa_label_admin.create_label
              (policy_name    => 'ESBD',
               label_tag      => 10,
               label_value    => 'EXEC:SALES,DEV,IS');
END;
/

BEGIN
  sa_label_admin.create_label
                      (policy_name    => 'ESBD',
                       label_tag      => 20,
                       label_value    => 'MGR:SALES');
  sa_label_admin.create_label
                         (policy_name    => 'ESBD',
                          label_tag      => 25,
                          label_value    => 'MGR:DEV');
END;
/

BEGIN
  sa_label_admin.create_label
                      (policy_name    => 'ESBD',
                       label_tag      => 30,
                       label_value    => 'EMP:SALES');
  sa_label_admin.create_label
                       (policy_name    => 'ESBD',
                        label_tag      => 35,
                        label_value    => 'EMP:DEV');
  sa_label_admin.create_label
                        (policy_name    => 'ESBD',
                         label_tag      => 39,
                         label_value    => 'EMP:IS');
END;
/

-- setup compartment and level authorizations
BEGIN
  sa_user_admin.add_compartments
                       (policy_name    => 'ESBD',
                        user_name      => 'ALL_EXECS',
                        comps          => 'SALES,DEV,IS');
END;
/

BEGIN
  sa_user_admin.set_user_labels
                  (policy_name       => 'ESBD',
                   user_name         => 'SALES_MANAGERS',
                   max_read_label    => 'MGR:SALES');
  sa_user_admin.set_user_labels
                  (policy_name       => 'ESBD',
                   user_name         => 'DEV_MANAGERS',
                   max_read_label    => 'MGR:DEV');
END;
/

BEGIN
  sa_user_admin.set_user_labels
                   (policy_name    => 'ESBD',
                    user_name      => 'SALES_EMPLOYEES',
                    max_read_label => 'EMP:SALES');
  sa_user_admin.set_user_labels
                   (policy_name    => 'ESBD',
                    user_name      => 'DEV_EMPLOYEES',
                    max_read_label => 'EMP:DEV');
  sa_user_admin.set_user_labels
                   (policy_name    => 'ESBD',
                    user_name      => 'INTERNAL_EMPLOYEES',
                    max_read_label => 'EMP:IS');
END;
/

-------------- inserting data -----------
conn scott/tiger
INSERT INTO scott.announcements
            (MESSAGE, ROWLABEL)
     VALUES ('New updates to quotas have been assigned.',
             char_to_label ('ESBD', 'MGR:SALES'));
INSERT INTO scott.announcements
            (MESSAGE, ROWLABEL)
     VALUES ('New product release date meeting scheduled.',
             char_to_label ('ESBD', 'MGR:DEV'));
INSERT INTO scott.announcements
            (MESSAGE, ROWLABEL)
     VALUES ('Quota club trip destined for Hawaii.',
             char_to_label ('ESBD', 'EMP:SALES'));
INSERT INTO scott.announcements
            (MESSAGE, ROWLABEL)
     VALUES ('Source control software updates distributed next week.',
             char_to_label ('ESBD', 'EMP:DEV'));
INSERT INTO scott.announcements
            (MESSAGE, ROWLABEL)
     VALUES ('Firewall attacks increasing.',
             char_to_label ('ESBD', 'EMP:IS'));
COMMIT ;

----------- testing the new compartments --------------
conn sec_mgr/oracle10g
COL message format a55
COL "OLS Label" format a10
-- If user authorizations do not include compartments, 
-- then no data is returned
begin
 sa_session.set_access_profile('ESBD','ALL_EMPLOYEES');
end;
/
SELECT MESSAGE FROM scott.announcements;

COL message format a55
-- Users with compartments see their compartments
-- and non-compartment labeled data.
-- Set authorization profile for sales employees
BEGIN
  sa_session.set_access_profile
                               ('ESBD',
                                'SALES_EMPLOYEES');
END;
/
SELECT MESSAGE
  FROM scott.announcements;
-- Managers can still see all employee data within
-- the same compartment.
COL "OLS Label" format a10
BEGIN
  sa_session.set_access_profile ('ESBD',
                                 'DEV_MANAGERS');
END;
/
-- Executives have access to all information.
BEGIN
  sa_session.set_access_profile ('ESBD',
                                 'ALL_EXECS');
END;
/
SELECT MESSAGE,
       label_to_char (ROWLABEL) "OLS Label"
  FROM scott.announcements;

------------------ Groups -----------------------
-- create the corporate group

BEGIN
  sa_components.CREATE_GROUP
                       (policy_name    => 'ESBD',
                        long_name      => 'Corporate',
                        short_name     => 'CORP',
                        group_num      => 1,
                        parent_name    => NULL);
END;
/

-- Create groups based on geographical regions
BEGIN
  -- Create group representing offices in the U.S.
  sa_components.CREATE_GROUP
                   (policy_name    => 'ESBD',
                    long_name      => 'United States',
                    short_name     => 'US',
                    group_num      => 100,
                    parent_name    => 'CORP');
  -- Create group representing the New York office.
  -- This is a child of the US group.
  sa_components.CREATE_GROUP
                  (policy_name    => 'ESBD',
                   long_name      => 'New York',
                   short_name     => 'NY',
                   group_num      => 110,
                   parent_name    => 'US');
  -- Create group representing the Los Angeles office.
  -- This is a child of the US group.
  sa_components.CREATE_GROUP
                  (policy_name    => 'ESBD',
                   long_name      => 'Los Angeles',
                   short_name     => 'LA',
                   group_num      => 120,
                   parent_name    => 'US');
END;
/

-- International
BEGIN
  sa_components.CREATE_GROUP
                 (policy_name    => 'ESBD',
                  long_name      => 'Europe Middle_East Africa',
                  short_name     => 'EMEA',
                  group_num      => 200,
                  parent_name    => 'CORP');
  sa_components.CREATE_GROUP
                 (policy_name    => 'ESBD',
                  long_name      => 'Asia and Pacific',
                  short_name     => 'APAC',
                  group_num      => 300,
                  parent_name    => 'CORP');
END;
/

-- Create labels with levels, compartments and groups
-- Sales in US and EMEA. 
-- Note US is divided into overall US, LA and then NY
BEGIN
  -- Sales managers for EMEA
  sa_label_admin.create_label
                 (policy_name    => 'ESBD',
                  label_tag      => 300,
                  label_value    => 'MGR:SALES:EMEA');
  -- Sales manager for all of U.S.
  sa_label_admin.create_label
                 (policy_name    => 'ESBD',
                  label_tag      => 310,
                  label_value    => 'MGR:SALES:US');
  -- New York sales reps
  sa_label_admin.create_label
                 (policy_name    => 'ESBD',
                  label_tag      => 320,
                  label_value    => 'EMP:SALES:NY');
  -- Los Angeles sales reps
  sa_label_admin.create_label
                 (policy_name    => 'ESBD',
                  label_tag      => 330,
                  label_value    => 'EMP:SALES:LA');
END;
/

-- Develop in US and APAC
-- Managers reside in corporate  facility only
BEGIN
  -- US developers
  sa_label_admin.create_label
                 (policy_name    => 'ESBD',
                  label_tag      => 400,
                  label_value    => 'EMP:DEV:US');
  -- APAC developers
  sa_label_admin.create_label
                 (policy_name    => 'ESBD',
                  label_tag      => 410,
                  label_value    => 'EMP:DEV:APAC');
  -- US developers
  sa_label_admin.create_label
                 (policy_name    => 'ESBD',
                  label_tag      => 450,
                  label_value    => 'MGR:DEV:CORP');
END;
/

--------------- create final auths ---------------
-- Setup level, compartment and group authorizations
-- add groups to executives. Only need to add root group
BEGIN
  sa_user_admin.add_groups
                 (policy_name    => 'ESBD',
                  user_name      => 'ALL_EXECS',
                  groups         => 'CORP');
END;
/

-- Create authorizations for US and EMEA sales managers
BEGIN
  sa_user_admin.set_user_labels
                (policy_name       => 'ESBD',
                 user_name         => 'US_SALES_MGR',
                 max_read_label    => 'MGR:SALES:US');
  sa_user_admin.set_user_labels
                (policy_name       => 'ESBD',
                 user_name         => 'EMEA_SALES_MGR',
                 max_read_label    => 'MGR:SALES:EMEA');
END;
/
-- Create authorizations for NY and LA sales reps
BEGIN
  sa_user_admin.set_user_labels
                (policy_name       => 'ESBD',
                 user_name         => 'NY_SALES_REP',
                 max_read_label    => 'EMP:SALES:NY');
  sa_user_admin.set_user_labels
                (policy_name       => 'ESBD',
                 user_name         => 'LA_SALES_REP',
                 max_read_label    => 'EMP:SALES:LA');
END;
/

-- Create authorizations for the development staff.
-- Add groups to managers. Only need to add root
BEGIN
  sa_user_admin.add_groups
                (policy_name    => 'ESBD',
                 user_name      => 'DEV_MANAGERS',
                 GROUPS         => 'CORP');
END;
/
-- Add developer profiles
BEGIN
  -- Create authorizations for APAC developers
  sa_user_admin.set_user_labels
                (policy_name       => 'ESBD',
                 user_name         => 'APAC_DEVELOPER',
                 max_read_label    => 'EMP:DEV:APAC');
  -- Create authorizations for US developers
  sa_user_admin.set_user_labels
                   (policy_name       => 'ESBD',
                    user_name         => 'US_DEVELOPER',
                    max_read_label    => 'EMP:DEV:US');
END;
/

-- Use default write session 
-- authorization for data label. Have to drop
-- then re-add policy with label_default option.
BEGIN
  sa_policy_admin.remove_table_policy
               (policy_name    => 'ESBD',
                schema_name    => 'SCOTT',
                table_name     => 'ANNOUNCEMENTS');
  sa_policy_admin.apply_table_policy
               (policy_name    => 'ESBD',
                schema_name    => 'SCOTT',
                table_name     => 'ANNOUNCEMENTS',
                table_options  => 'LABEL_DEFAULT,READ_CONTROL');
END;
/

-- Insert data as a US sales manager. OLS will
-- automatically label data based on user's write label.
BEGIN
  sa_session.set_access_profile ('ESBD',
                                 'US_SALES_MGR');
END;
/
INSERT INTO scott.announcements
            (MESSAGE)
     VALUES ('Presidential outlook for economy may affect revenue.');
COMMIT ;
-- Check label of inserted message
SELECT MESSAGE,
       label_to_char (ROWLABEL) "OLS Label"
  FROM scott.announcements
 WHERE MESSAGE LIKE 'Pres%';


-- Insert sales data using session label.
BEGIN
-- Insert data as NY sales rep
  sa_session.set_access_profile ('ESBD',
                                 'NY_SALES_REP');
  INSERT INTO scott.announcements
              (MESSAGE)
       VALUES ('Party in Madison Ave. office cancelled');
-- Insert data as LA sales rep
  sa_session.set_access_profile ('ESBD',
                                 'LA_SALES_REP');
  INSERT INTO scott.announcements
              (MESSAGE)
       VALUES ('Earthquake preparation team meeting tonight.');
-- Insert data as APAC developer sales rep
  sa_session.set_access_profile ('ESBD',
                                 'APAC_DEVELOPER');
  INSERT INTO scott.announcements
              (MESSAGE)
       VALUES ('National Language Support API released.');
  COMMIT;
END;
/

--------- TESTING ------------
COL message format a55
COL "OLS Label" format a12
BEGIN
  sa_session.set_access_profile ('ESBD',
                                 'ALL_EXECS');
END;
/
SELECT MESSAGE,
       label_to_char (ROWLABEL) "OLS Label"
  FROM scott.announcements;

EXEC sa_session.set_access_profile('ESBD','NY_SALES_REP');
/
EXEC sa_session.set_access_profile('ESBD','LA_SALES_REP');
/
EXEC sa_session.set_access_profile('ESBD','US_SALES_MGR');
/
EXEC sa_session.set_access_profile('ESBD','EMEA_SALES_MGR');
/
EXEC sa_session.set_access_profile('ESBD','APAC_DEVELOPER');
/
EXEC sa_session.set_access_profile('ESBD','DEV_MANAGERS');
/
-- Label Dominance
COL "User's Read Label" format a20
COL "Data Record Labels" format a20
BREAK on "User's Read Label"
BEGIN
  sa_session.set_access_profile ('ESBD',
                                 'US_SALES_MGR');
END;
/
SELECT   sa_session.read_label ('ESBD')
                              "User's Read Label",
         label_to_char (ROWLABEL)
                              "Data Record Labels"
    FROM scott.announcements
GROUP BY ROWLABEL;
EXEC sa_session.set_access_profile('ESBD','APAC_DEVELOPER');
/

------------------------ Hiding the column ----------
CONN sec_mgr/oracle10g
-- Drop OLS policy.
BEGIN
  sa_policy_admin.remove_table_policy
                   (policy_name    => 'ESBD',
                    schema_name    => 'SCOTT',
                    table_name     => 'ANNOUNCEMENTS');
END;
/
-- Add Policy with Hide option. 
BEGIN
  sa_policy_admin.apply_table_policy
    (policy_name      => 'ESBD',
     schema_name      => 'SCOTT',
     table_name       => 'ANNOUNCEMENTS',
     table_options    => 'HIDE,LABEL_DEFAULT,READ_CONTROL');
END;
/

CONN scott/tiger
-- Add temporary column to table
ALTER TABLE announcements ADD (temp_label NUMBER(10));


CONN sec_mgr/oracle10g

-- Set access to all rows
BEGIN
  sa_session.set_access_profile ('ESBD',
                                 'ALL_EXECS');
END;
/
-- Copy labels to temporary column
UPDATE scott.announcements
   SET temp_label = ROWLABEL;
COMMIT ;

-- Verify update
COL message format a55
COL "OLS Label" format a12
SELECT rownum,
       label_to_char (ROWLABEL) "OLS Label",
       rowlabel "Label Value",
       temp_label "Temp Label Value"
  FROM scott.announcements;

-- Remove policy and re-apply with HIDE option
BEGIN
  sa_policy_admin.remove_table_policy
    (policy_name    => 'ESBD',
     schema_name    => 'SCOTT',
     table_name     => 'announcements',
     DROP_COLUMN    => TRUE);
  sa_policy_admin.apply_table_policy
    (policy_name      => 'ESBD',
     schema_name      => 'SCOTT',
     table_name       => 'ANNOUNCEMENTS',
     table_options    => 'HIDE,LABEL_DEFAULT,READ_CONTROL');
                    
END;
/

-- Copy values back to rowlabel column
UPDATE scott.announcements
   SET ROWLABEL = temp_label;
COMMIT ;
-- Verify update
SELECT ROWNUM,
       label_to_char (ROWLABEL) "OLS Label",
       ROWLABEL "Label Value",
       temp_label "Temp Label Value"
  FROM scott.announcements;

-- Drop temporary column
conn scott/tiger
ALTER TABLE announcements DROP COLUMN temp_label;

------------------ Write Controls -------------------

CONN sec_mgr/oracle10g
-- Create scott's authorizations.
-- Give a different read level and insert level
BEGIN
  sa_user_admin.set_levels
                 (policy_name    => 'ESBD',
                  user_name      => 'SCOTT',
                  max_level      => 'MGR',
                  min_level      => 'EMP',
                  def_level      => 'MGR',
                  row_level      => 'EMP');
END;
/

CONN scott/tiger
-- Show Scott's labels
COL "Select Label" format a15
COL "Insert Label" format a15
SELECT sa_session.read_label ('ESBD') "Select Label",
       sa_session.row_label ('ESBD') "Insert Label"
  FROM DUAL;

-- Insert data. Data is automatically labeled 
-- to the row_level value
INSERT INTO announcements
     VALUES ('Scott will be out of the office.');
COL "OLS Label" format a12
COL message     format a50
SELECT MESSAGE, label_to_char (ROWLABEL) "OLS Label"
  FROM announcements
 WHERE MESSAGE LIKE 'Scott%';

-- Show default labels user can access
COL "User Read Label" format a20
COL "Data Record Labels" format a20
BREAK on "User's Read Label"
SELECT sa_session.read_label ('ESBD') "User Read Label",
       label_to_char (ROWLABEL) "Data Record Labels"
    FROM scott.announcements
GROUP BY ROWLABEL;
-- Change the user's default read label
EXEC sa_session.set_label('ESBD','EMP');
/

-- This fails if session label is set to EMP
EXEC sa_session.set_row_label('ESBD','MGR');
-- Reset session and try again
EXEC sa_session.set_label('ESBD','MGR');
EXEC sa_session.set_row_label('ESBD','MGR');
INSERT INTO announcements
     VALUES ('Scott''s management meeting is cancelled.');
COL "OLS Label" format a12
COL message     format a51
SELECT MESSAGE, label_to_char (ROWLABEL) "OLS Label"
  FROM announcements;
COMMIT ;

-------------------- Compartment/Group dependency --------------
/***************
If the label has no groups, then the user must have write access on all the
compartments in the label, in order to write the data.

If the label does have groups, and the user has write access to one of the groups,
she only needs read access to the compartments, in order to write the data
****************/

DECLARE
  l_options  VARCHAR2 (50)
    := 'HIDE,LABEL_DEFAULT,WRITE_CONTROL,READ_CONTROL';
BEGIN
  sa_policy_admin.remove_table_policy
                   (policy_name    => 'ESBD',
                    schema_name    => 'SCOTT',
                    table_name     => 'ANNOUNCEMENTS');
  sa_policy_admin.apply_table_policy
                   (policy_name      => 'ESBD',
                    schema_name      => 'SCOTT',
                    table_name       => 'ANNOUNCEMENTS',
                    table_options    => l_options);
END;
/

COL "OLS Label" format a12
COL message format a63
COL "Read Label" format a30
COL "Write Label" format a30

CONN sec_mgr/oracle10g
EXEC sa_session.set_access_profile('ESBD','ALL_EXECS');
SELECT MESSAGE,
       label_to_char (ROWLABEL) "OLS Label"
  FROM scott.announcements;
-- turn on write control


-- Add DEV compartment to SCOTT 
-- for read/write operations
BEGIN
  sa_user_admin.add_compartments
               (policy_name    => 'ESBD',
                user_name      => 'SCOTT',
                comps          => 'DEV',
                access_mode    => sa_utl.read_write,
                in_def         => 'Y',
                in_row         => 'Y');
END;
/

-- Add SALES compartment for SCOTT as read only.
BEGIN
  sa_user_admin.add_compartments
                (policy_name    => 'ESBD',
                 user_name      => 'SCOTT',
                 comps          => 'SALES',
                 access_mode    => sa_utl.read_only,
                 in_def         => 'Y',
                 in_row         => 'N');
END;
/

-- Add US group to Scott for read/write operations
-- both as a default label and as insert/update label. 
BEGIN
  sa_user_admin.add_groups
               (policy_name    => 'ESBD',
                user_name      => 'SCOTT',
                GROUPS         => 'US',
                access_mode    => sa_utl.read_write,
                in_def         => 'Y',
                in_row         => 'Y');
END;
/

-- Show final authorizations for Scott
CONN scott/tiger
COL "OLS Label" format a12
COL "Read Label" format a30
COL "Write Label" format a30
-- Show user's read and write labels
SELECT sa_session.read_label ('ESBD') "Read Label",
       sa_session.write_label ('ESBD') "Write Label"
  FROM DUAL;
-- Show how labels are spread across data in table  
SELECT label_to_char (ROWLABEL) "OLS Label",
       COUNT (ROWLABEL) "Total Records Labeled"
    FROM announcements
GROUP BY ROWLABEL;

-- Delete data labeled with SALES compartment. 
DELETE FROM announcements
      WHERE label_to_char (ROWLABEL) LIKE '%:SALES:%';
SELECT MESSAGE,
       label_to_char (ROWLABEL) "OLS Label"
  FROM scott.announcements;
SELECT label_to_char (ROWLABEL) "OLS Label",
       COUNT (ROWLABEL) "Total Records Labeled"
    FROM announcements
GROUP BY ROWLABEL;
ROLLBACK ;

-------- Restricting Updates ----------------
COL "OLS Label" format a12
COL message format a50
COL "Read Label" format a30
COL "Write Label" format a30

CONN sec_mgr/oracle10g
-- Enable label_update enforcement
BEGIN
  sa_policy_admin.remove_table_policy
                   (policy_name    => 'ESBD',
                    schema_name    => 'SCOTT',
                    table_name     => 'ANNOUNCEMENTS');
  sa_policy_admin.apply_table_policy
    (policy_name      => 'ESBD',
     schema_name      => 'SCOTT',
     table_name       => 'ANNOUNCEMENTS',
     table_options    =>    'HIDE,LABEL_DEFAULT,LABEL_UPDATE,'
                         || 'WRITE_CONTROL,READ_CONTROL');
END;
/

CONN scott/tiger
-- User can update certain rows
UPDATE announcements
   SET MESSAGE = INITCAP (MESSAGE)
 WHERE label_to_char (ROWLABEL) like '%DEV%';
-- User is not authorized to change the security label
UPDATE announcements
   SET ROWLABEL = char_to_label ('ESBD', 'EMP')
 WHERE label_to_char (ROWLABEL) like '%DEV%';

-- Grant writedown privilege to SCOTT
BEGIN
  sa_user_admin.set_user_privs
                 (policy_name    => 'ESBD',
                  user_name      => 'SCOTT',
                  PRIVILEGES     => 'WRITEDOWN');
END;
/

-- Have to reset the authorizations be re-enabling 
-- the profile. This is done automatically on logon.
CONN scott/tiger
-- Reclassify data making it available to all
-- developers
UPDATE announcements
   SET ROWLABEL = char_to_label ('ESBD', 'EMP:DEV')
 WHERE label_to_char (ROWLABEL) = 'MGR:DEV'
/
ROLLBACK ;

--------------- Trusted Procedures ----------------
CONN scott/tiger
-- Create function to return number of messages.
CREATE OR REPLACE FUNCTION get_quantity (
  p_message  IN  VARCHAR2)
  RETURN PLS_INTEGER
AS
  l_total  PLS_INTEGER;
BEGIN
  SELECT COUNT (*)
    INTO l_total
    FROM announcements
   WHERE UPPER (MESSAGE) LIKE
                   '%' || UPPER (p_message)
                   || '%';
  RETURN l_total;
END;
/
-- 
SELECT get_quantity ('scott') FROM DUAL;
GRANT EXECUTE ON get_quantity TO blake;

CONN blake/blake;
-- User has no authorizations, so will see no data
SELECT scott.get_quantity ('scott') FROM DUAL;

CONN sec_mgr/sec_mgr
-- Authorize the procedure
BEGIN
  sa_user_admin.set_prog_privs
             (policy_name          => 'ESBD',
              schema_name          => 'SCOTT',
              program_unit_name    => 'get_quantity',
              PRIVILEGES           => 'READ');
END;
/
conn blake/blake;
-- user has no auth, but procedure is authorized
select scott.get_quantity('resident') from dual;

----------------- Label Functions ------------------
-- Create a label function.
CREATE OR REPLACE FUNCTION gen_label (
  p_message  VARCHAR2)
  RETURN lbacsys.lbac_label
AS
  l_label  VARCHAR (80);
BEGIN
  -- If label contains "manage" (assuming structured format)
  -- then label as MGR. If label contains "employee"
  -- then must be EMP data. Otherwise label with EXEC
  CASE
    WHEN UPPER (p_message) LIKE '%MANAGE%'
    THEN
      l_label := 'MGR';
    WHEN UPPER (p_message) LIKE '%EMPLOYEE%'
    THEN
      l_label := 'EMP';
    ELSE
      l_label := 'EXEC';
  END CASE;
  -- If authorization profile is SCOTT, then 
  -- add compartment DEV. Otherwise label as sales.
  CASE
    WHEN sa_session.sa_user_name('ESBD') = 'SCOTT'
    THEN
      l_label := l_label || ':DEV';
    ELSE
      l_label := l_label || ':SALES';
  END CASE;

  RETURN to_lbac_data_label ('ESBD', l_label);
END;
/

-- Apply label function
BEGIN
  sa_policy_admin.remove_table_policy
                   (policy_name    => 'ESBD',
                    schema_name    => 'SCOTT',
                    table_name     => 'ANNOUNCEMENTS');
  sa_policy_admin.apply_table_policy
    (policy_name      => 'ESBD',
     schema_name      => 'SCOTT',
     table_name       => 'ANNOUNCEMENTS',
     label_function   => 'SEC_MGR.GEN_LABEL(:new.message)',
     table_options    =>    'HIDE,READ_CONTROL,WRITE_CONTROL');
END;
/

conn scott/tiger
-- Insert an employee record
INSERT INTO announcements
            (MESSAGE)
     VALUES ('All employees should call in sick tomorrow.');
-- Insert a manager record
INSERT INTO announcements
            (MESSAGE)
     VALUES ('All managers are required to work late.');
COMMIT;

CONN sec_mgr/oracle10g
BEGIN
  sa_session.set_access_profile ('ESBD',
                                 'ALL_EXECS');
END;
/
-- Insert record as executive.
INSERT INTO scott.announcements
            (MESSAGE)
     VALUES ('Stock split announcement tonight');
-- Verify labels
COL "OLS Label" format a12
COL message format a51
SELECT MESSAGE,
       label_to_char (ROWLABEL) "OLS Label"
  FROM scott.announcements
 WHERE MESSAGE LIKE 'All%'
       OR MESSAGE LIKE 'Stock%';
