
DROP TABLE authorized_users;
drop table lookup_dept;
CREATE TABLE lookup_dept
AS SELECT ename username, deptno FROM emp;

DROP CONTEXT ctx_ex;
CREATE CONTEXT ctx_ex USING sec_mgr.ctx_ex_mgr;

CREATE OR REPLACE PACKAGE ctx_ex_mgr
AS
  PROCEDURE set_deptno;
  PROCEDURE clear_deptno;
END;
/

CREATE OR REPLACE PACKAGE BODY ctx_ex_mgr
AS
-----------------------------------------
  PROCEDURE set_deptno
  AS
    l_deptno  NUMBER;
  BEGIN
    SELECT deptno
      INTO l_deptno
      FROM lookup_dept
     WHERE username =
             SYS_CONTEXT ('userenv',
                          'session_user');
    DBMS_SESSION.set_context
                           (namespace    => 'ctx_ex',
                            ATTRIBUTE    => 'deptno',
                            VALUE        => l_deptno);
  END set_deptno;

-----------------------------------------
  PROCEDURE clear_deptno
  AS
  BEGIN
    DBMS_SESSION.clear_context
                          (namespace    => 'ctx_ex',
                           ATTRIBUTE    => 'deptno');
  END clear_deptno;
-----------------------------------------
END ctx_ex_mgr;
/

CREATE OR REPLACE TRIGGER set_user_deptno
  AFTER LOGON ON DATABASE
BEGIN
  sec_mgr.ctx_ex_mgr.set_deptno;
EXCEPTION
  WHEN NO_DATA_FOUND
  THEN
    -- user is not in our table
    -- so the value will not be set.
    -- If exception is not handled, then some users
    -- may be unable to log on
    NULL;
END;
/

conn scott/tiger
COL deptno format a10
SELECT SYS_CONTEXT ('ctx_ex', 'deptno') deptno
  FROM DUAL;

CREATE OR REPLACE VIEW my_dept_ctx
AS
  SELECT *
    FROM emp
   WHERE deptno = SYS_CONTEXT ('ctx_ex', 'deptno');
-- View restricts records based on value of
-- application context
SELECT ename, deptno FROM my_dept_ctx;

-- create a trigger which prevents a user from
-- modifying data outside their department
CREATE OR REPLACE TRIGGER restrict_updates
  BEFORE DELETE OR UPDATE
  ON emp
  FOR EACH ROW
BEGIN
  IF (:OLD.deptno !=
                  SYS_CONTEXT ('ctx_ex', 'deptno'))
  THEN
    raise_application_error
      (-20001,
          CHR (10)
       || '**  You can only update records within your department.'
       || CHR (10)
       || '**  Your department number is '
       || SYS_CONTEXT ('ctx_ex', 'deptno'));
  END IF;
END;
/
UPDATE emp
   SET ename = ename
 WHERE deptno = 10;
DELETE FROM emp;

-------------------- show performance of trigger using application context ----
SET timing on
-- issue authorized updates 
-- each update will cause trigger to fire
BEGIN
  FOR i IN 1 .. 100000
  LOOP
    UPDATE emp
       SET ename = ename
     WHERE deptno = 20;
  END LOOP;
END;
/

set timing off
-- modify trigger to use select instead of context
CREATE OR REPLACE TRIGGER restrict_updates
  BEFORE DELETE OR UPDATE
  ON emp
  FOR EACH ROW
DECLARE
  l_deptno  NUMBER;
BEGIN
  SELECT deptno
    INTO l_deptno
    FROM lookup_dept
   WHERE ename = USER;

  IF (:OLD.deptno != l_deptno)
  THEN
    raise_application_error
      (-20001,
          CHR (10)
       || '**  You can only update records within your department.'
       || CHR (10)
       || '**  Your department number is '
       || l_deptno);
  END IF;
END;
/
-- test
UPDATE emp
   SET ename = ename
 WHERE deptno = 10;
DELETE FROM emp;

SET timing on
-- time new trigger implementation based on SQL query
BEGIN
  FOR i IN 1 .. 100000
  LOOP
    UPDATE emp
       SET ename = ename
     WHERE deptno = 20;
  END LOOP;
END;
/


--------------------- Show perf of application context -----
-- create view example
conn scott/tiger
CREATE VIEW my_dept_sql
AS
  SELECT *
    FROM emp
   WHERE deptno = (SELECT deptno
                     FROM emp
                    WHERE ename = USER);
-- show it works
SELECT *
  FROM my_dept_sql;
-- test performance
SET timing on

DECLARE
  l_total  NUMBER;
BEGIN
  FOR i IN 1 .. 1000000
  LOOP
    SELECT COUNT (*)
      INTO l_total
      FROM my_dept_sql;
  END LOOP;
END;
/

-- change view to use app context
CREATE OR REPLACE VIEW my_dept_ctx
AS
  SELECT *
    FROM emp
   WHERE deptno = SYS_CONTEXT ('ctx_ex', 'deptno');
-- show it works
SELECT *
  FROM my_dept_ctx;
-- test performance
SET timing on

DECLARE
  l_total  NUMBER;
BEGIN
  FOR i IN 1 .. 1000000
  LOOP
    SELECT COUNT (*)
      INTO l_total
      FROM my_dept_ctx;
  END LOOP;
END;
/



-- refernece context, first without it being set
-- note, that errors not thrown
DESCRIBE sec_mgr.ctx_ex_mgr
COL deptno format a10
SELECT SYS_CONTEXT ('ctx_ex', 'deptno') deptno
  FROM DUAL;

-- set context for demo
EXEC ctx_ex.set_deptno

-- show results
SELECT SYS_CONTEXT ('ctx_ex', 'deptno')
  FROM DUAL;

-- show the results now for scott
@conn scott/tiger
-- refernece context, first without it being set
-- note, that errors not thrown
SELECT SYS_CONTEXT ('ctx_ex', 'deptno')
  FROM DUAL;

--------------------- Typos problem ---------------
-- database errors with undefined attribute
SELECT SYS_CONTEXT ('userenv', 'foo') FROM DUAL;
-- no errors for users undefined attribute
SELECT SYS_CONTEXT ('ctx_ex', 'dptno') FROM DUAL;

CREATE OR REPLACE PACKAGE ctx_ex_mgr
AS
  namespace  CONSTANT VARCHAR2 (6) := 'CTX_EX';
  deptno     CONSTANT VARCHAR2 (6) := 'DEPTNO';
  PROCEDURE set_deptno;
  PROCEDURE clear_deptno;
END;
/
CREATE OR REPLACE PACKAGE BODY ctx_ex_mgr
AS
-----------------------------------------
  PROCEDURE set_deptno
  AS
    v_deptno  NUMBER;
    BEGIN
      SELECT deptno
        INTO v_deptno
        FROM authorized_users
       WHERE username =
               SYS_CONTEXT ('userenv',
                            'session_user');
      DBMS_SESSION.set_context
               (namespace    => ctx_ex_mgr.namespace,
                ATTRIBUTE    => ctx_ex_mgr.deptno,
                VALUE        => v_deptno);
  END set_deptno;
-----------------------------------------
  PROCEDURE clear_deptno
  AS
  BEGIN
    DBMS_SESSION.clear_context
              (namespace    => ctx_ex_mgr.namespace,
               ATTRIBUTE    => ctx_ex_mgr.deptno);
  END clear_deptno;
-----------------------------------------
END ctx_ex_mgr;
/
-- done for showing how to call from outside package
GRANT EXECUTE ON ctx_ex_mgr TO scott;

conn scott/tiger
drop synonym ctx_ex_mgr;
CREATE SYNONYM ctx_ex_mgr FOR sec_mgr.ctx_ex_mgr;
SELECT SYS_CONTEXT (ctx_ex_mgr.namespace,
                    ctx_ex_mgr.deptno)
  FROM DUAL;

conn system/manager
drop context ctx_ex;
CREATE CONTEXT ctx_ex USING ctx_mgr;
CONN sec_mgr/oracle10g
-- in wrong schema
CREATE OR REPLACE PROCEDURE ctx_mgr
AS
BEGIN
  DBMS_SESSION.set_context
                  (namespace    => 'CTX_EX',
                   ATTRIBUTE    => 'favorite_color',
                   VALUE        => 'blue');
END;
/
-- no errors, so you might assume everything is fine
EXEC ctx_mgr;

COL schema format a15
SELECT SCHEMA, PACKAGE, TYPE
  FROM dba_context
 WHERE namespace = 'CTX_EX';


CONN sec_mgr/sec-Mgr

CREATE OR REPLACE PROCEDURE ctx_mgr (
  p_favorite_color  IN  VARCHAR2)
AS
BEGIN
  DBMS_SESSION.ctx_ex
                  (namespace    => 'CTX_EX',
                   ATTRIBUTE    => 'favorite_color',
                   VALUE        => p_favorite_color);
END;
/


SELECT SYS_CONTEXT ('userenv',
                    'global_context_memory')
  FROM DUAL;









































































































































































